import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:hub/components/atomic_components/shared_components_atoms/custom_input.dart';
import 'package:hub/components/atomic_components/shared_components_atoms/submit_button.dart';
import 'package:hub/components/atomic_components/shared_components_atoms/tabview.dart';
import 'package:hub/components/templates_components/card_item_template_component/card_item_template_component_widget.dart';
import 'package:hub/features/backend/index.dart';
import 'package:hub/flutter_flow/flutter_flow_icon_button.dart';
import 'package:hub/flutter_flow/flutter_flow_theme.dart';
import 'package:hub/flutter_flow/flutter_flow_util.dart';
import 'package:hub/pages/vehicles_on_the_property/vehicle_model.dart';
import 'package:hub/shared/utils/dialog_util.dart';
import 'package:hub/shared/utils/limited_text_size.dart';
import 'package:hub/shared/utils/log_util.dart';
import 'package:material_symbols_icons/symbols.dart';

part 'vehicle_history_screen.dart';
part 'vehicle_register_screen.dart';
part 'vehicle_update_screen.dart';

/// [VehiclePage] is a StatefulWidget that displays the vehicle screens.

class VehiclePage extends StatefulWidget {
  const VehiclePage({super.key});

  @override
  _VehiclePageState createState() => _VehiclePageState();
}

class _VehiclePageState extends State<VehiclePage> with TickerProviderStateMixin {
  int count = 0;
  late final VehicleModel _model;

  @override
  void initState() {
    super.initState();
    _model = createModel(context, () => VehicleModel());

    _model.updateOnChange = true;
    _model.onUpdateVehicle = () {
      safeSetState(() {
        _model.clearFields();
      });
    };
    _model.onRegisterVehicle = () {
      safeSetState(() {
        _model.clearFields();
      });
    };
    _model.safeSetState = () {
      safeSetState(() {});
    };
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final backgroundColor = FlutterFlowTheme.of(context).primaryBackground;
    return Scaffold(
      backgroundColor: backgroundColor,
      appBar: _buildAppBar(context),
      body: _buildBody(context),
    );
  }

  /// [Body] of the page.
  void onEditingChanged(bool value) {
    setState(() {
      _model.handleEditingChanged(value);
    });
  }

  Widget _buildBody(BuildContext context) {
    final vehicleHistoryScreenLabel =
        FFLocalizations.of(context).getVariableText(ptText: 'Consultar', enText: 'Consult');
    final vehicleRegisterScreenLabel =
        FFLocalizations.of(context).getVariableText(ptText: 'Cadastrar', enText: 'Register');
    final vehicleUpdateScreenLabel = FFLocalizations.of(context).getVariableText(ptText: 'Editar', enText: 'Edit');

    return TabViewUtil(
      context: context,
      model: _model,
      labelTab1: vehicleHistoryScreenLabel,
      labelTab2: _model.isEditing ? vehicleUpdateScreenLabel : vehicleRegisterScreenLabel,
      controller: _model.tabBarController,
      widget1: VehicleHistoryScreen(_model),
      widget2: _model.isEditing ? VehicleUpdateScreen(_model) : VehicleRegisterScreen(_model),
      onEditingChanged: onEditingChanged,
    );
  }

  /// -----------------------------------
  /// [AppBar] with the title of the page.
  PreferredSizeWidget _buildAppBar(BuildContext context) {
    final theme = FlutterFlowTheme.of(context);
    final backgroundColor = theme.primaryBackground;
    final primaryText = theme.primaryText;
    final title = FFLocalizations.of(context).getVariableText(enText: 'Vehicles', ptText: 'Veículos');
    final titleStyle = theme.headlineMedium.override(
      fontFamily: theme.headlineMediumFamily,
      color: primaryText,
      fontSize: 16.0,
      fontWeight: FontWeight.bold,
      letterSpacing: 0.0,
      useGoogleFonts: GoogleFonts.asMap().containsKey(theme.headlineMediumFamily),
    );
    final backButton = _backButton(context, theme);

    return AppBar(
      backgroundColor: backgroundColor,
      automaticallyImplyLeading: false,
      title: Text(title, style: titleStyle),
      leading: backButton,
      centerTitle: true,
      elevation: 0.0,
      actions: [],
    );
  }

  Widget _backButton(BuildContext context, FlutterFlowTheme theme) {
    final Icon icon = Icon(
      Icons.keyboard_arrow_left,
      color: theme.primaryText,
      size: 30.0,
    );
    onPressed() => Navigator.of(context).pop();
    return FlutterFlowIconButton(
      key: ValueKey<String>('BackNavigationAppBar'),
      borderColor: Colors.transparent,
      borderRadius: 30.0,
      borderWidth: 1.0,
      buttonSize: 60.0,
      icon: icon,
      onPressed: onPressed,
    );
  }

  /// -----------------------------------
}
